"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.signMessage = exports.importKeyFromPem = exports.sign = exports.exportKeyToPem = exports.generateKeyPair = exports.Bytes = void 0;
const crypto = require("node:crypto");
const chunk = (arr, size) => {
    const pieces = Math.ceil(arr.length / size);
    return new Array(pieces).fill([]).map((_, i) => arr.slice(i * size, (i * size) + size));
};
class Bytes {
    constructor(bytes) {
        this._bytes = bytes;
        this.length = bytes.length;
    }
    array() {
        return this._bytes;
    }
    utf8() {
        return new TextDecoder().decode(this._bytes);
    }
    base64() {
        const str = String.fromCharCode(...this._bytes);
        return btoa(str);
    }
    arrayBuffer() {
        return this._bytes.buffer.slice(0);
    }
    async sha256() {
        const hash = await crypto.webcrypto.subtle.digest('SHA-256', this._bytes);
        return new Bytes(new Uint8Array(hash));
    }
    static fromBase64(str) {
        return new Bytes(Buffer.from(str, 'base64'));
    }
    static fromUtf8(str) {
        return new this(new TextEncoder().encode(str));
    }
    static fromArrayBuffer(buffer) {
        return new this(new Uint8Array(buffer));
    }
    static from(obj, encoding = "utf-8") {
        if (obj instanceof String) {
            if (encoding === "utf-8") {
                return Bytes.fromUtf8(obj);
            }
            else if (encoding === "base64") {
                return Bytes.fromBase64(obj);
            }
        }
        else if (obj instanceof ArrayBuffer) {
            return Bytes.fromArrayBuffer(obj);
        }
        return new Bytes(new Uint8Array());
    }
}
exports.Bytes = Bytes;
const generateKeyPair = async () => {
    return await crypto.webcrypto.subtle.generateKey({
        name: 'RSASSA-PKCS1-v1_5',
        modulusLength: 2048,
        publicExponent: new Uint8Array([0x01, 0x00, 0x01]),
        hash: { name: 'SHA-256' },
    }, true, // extractable
    ['sign', 'verify']);
};
exports.generateKeyPair = generateKeyPair;
const exportKeyToPem = async (key, type) => {
    const exported = Bytes.fromArrayBuffer(await crypto.webcrypto.subtle.exportKey(type === 'private' ? 'pkcs8' : 'spki', key));
    const b64 = exported.base64();
    const typeUpper = type.toUpperCase();
    return [`-----BEGIN ${typeUpper} KEY-----`, ...chunk([...b64], 64).map(v => v.join('')), `-----END ${typeUpper} KEY-----`].join('\n');
};
exports.exportKeyToPem = exportKeyToPem;
const sign = async (key, data) => {
    const buffer = await crypto.webcrypto.subtle.sign('RSASSA-PKCS1-v1_5', key, data.array());
    return Bytes.from(buffer);
};
exports.sign = sign;
const importKeyFromPem = async (pemText, type) => {
    pemText = pemText.trim();
    const typeUpper = type.toUpperCase();
    const b64 = pemText.substring(`-----BEGIN ${typeUpper} KEY-----`.length, pemText.length - `-----END ${typeUpper} KEY-----`.length).replace(/\s+/g, '');
    const pemBytes = Bytes.fromBase64(b64);
    return await crypto.webcrypto.subtle.importKey(type === 'private' ? 'pkcs8' : 'spki', pemBytes.array(), {
        name: 'RSASSA-PKCS1-v1_5',
        hash: 'SHA-256',
    }, false, // extractable
    [type === 'private' ? 'sign' : 'verify']);
};
exports.importKeyFromPem = importKeyFromPem;
const signMessage = async (key, message) => {
    const signed = await (0, exports.sign)(key, Bytes.fromUtf8(message));
    return signed.base64();
};
exports.signMessage = signMessage;
